import functools
import inspect
from vvecon.rest_api.utils.Types import CURSOR, ANY
from vvecon.rest_api.utils.Abort import Abortion

__all__ = ("Controller", "cover")


def cover(error: str):
    """
    This function will cover the error handling part on Controllers
    :param error: Error message to be sent out from the API
    :return: Returns an object or whatever returning from the decorated function
    """
    def func(f):
        """
        This method will wrap the desired function with a wrapping tool
        :param f: Function to be wrapped around
        :return: Returns an object for whatever returning from the decorated function
        """
        @functools.wraps(f)
        def wrapper(self, *args, **kwargs):
            """
            This wrapper function will cover the function with base flow of the process
            :param self: Controller itself
            :param args: arguments to be passed into the function
            :param kwargs: keywords to be passed into the function
            :return: Returns whatever returning from the decorated function
            """
            with self.db as con:
                cursor: CURSOR = con.cursor()
                try:
                    con_required = 'con' in inspect.signature(f).parameters
                    if con_required:
                        return f(self, *args, **kwargs, con=con, cursor=cursor)
                    if not con_required:
                        return f(self, *args, **kwargs, cursor=cursor)
                except Exception as e:
                    print("[FAILED: " + error.upper() + "]", e)
                    return Abortion("SYSTEM:"+error.lower())
                finally:
                    cursor.close()
                    con.close()

        return wrapper

    return func


# -- Controller -- #
class Controller:
    # To be defined
    MAIL = None

    @staticmethod
    def check(result: ANY, check: ANY = None) -> ANY:
        return result == check or type(result) == Abortion

    @staticmethod
    def failed(result: ANY, message: str = "Operation failed", code: int = 404) -> ANY:
        return result if type(result) == Abortion else Abortion(message, code=code)

    def send_email(self, recipients: list, subject: str = "", body: str = ""):
        try:
            return self.__class__.MAIL.send(recipients, subject, body)
        except Exception as e:
            print("[MAIL FAILED: ", e, "]")
            return Abortion("VRAError 007: Failed to send the mail")



from datetime import datetime
from ftplib import FTP
from dotenv import load_dotenv
from os import environ

load_dotenv("../.env")

# FTP server credentials and file information
FTP_URL = environ.get("FTP_URL")
FTP_BASE = environ.get("FTP_BASE")
FTP_HOST = environ.get("FTP_HOST")
FTP_USER = environ.get("FTP_USER")
FTP_PASSWORD = environ.get("FTP_PASSWORD")
LOCAL_BASE = environ.get("LOCAL_BASE")
FILES = ['tmp/restart.txt']

if __name__ == "__main__":
    try:

        print("CONNECTING")

        # Create an FTP connection
        ftp = FTP(FTP_HOST)

        # Login to the FTP server
        ftp.login(user=FTP_USER, passwd=FTP_PASSWORD)

        print("CONNECTED TO THE SERVER")


        def upload(path):
            try:
                current_time = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
                file_content = f"Uploaded at {current_time}"

                temp_file_path = f"{LOCAL_BASE}{path}"
                with open(temp_file_path, "w") as temp_file:
                    temp_file.write(file_content)

                ftp.storbinary("STOR " + FTP_BASE + path, open(temp_file_path, "rb"))

            except Exception as err:
                print(f"Upload failed for {path} due to:\n{err}")


        print("RESTARTING")
        for FILE in FILES:
            upload(FILE)

        print("RESTART COMPLETED")
        # Close the FTP connection
        ftp.quit()

    except Exception as e:
        print("RESTART FAILED\nDUE TO\n", e)



from flask_socketio import SocketIO as socketIO


class SocketIO(socketIO):
    MODE = None

    def __init__(self, *args, **kwargs):
        super(SocketIO, self).__init__(*args, **kwargs)

    def init_app(self, app, **kwargs):
        if "async_mode" in kwargs:
            self.__class__.MODE = kwargs["async_mode"]
        super(SocketIO, self).init_app(app, **kwargs)

    def run(self, app, host: str = "0.0.0.0", debug: bool = False, port: int = 8000, **kwargs):
        if self.__class__.MODE == "eventlet":
            import eventlet
            eventlet.wsgi.server(eventlet.listen((host, port)), self)
            return
        if "allow_unsafe_werkzeug" in kwargs:
            kwargs.pop("allow_unsafe_werkzeug")
        super(SocketIO, self).run(debug=debug, port=port, allow_unsafe_werkzeug=True, **kwargs)



from vvecon.rest_api import App
from Resources import User, sock, mail
from dotenv import load_dotenv
from os import environ

load_dotenv()

# initializing API
app = App(__name__)
app.mail(mail, environ.get("EMAIL"), environ.get("APP_PASSWORD"))
app.socket(sock)

# add Admin class to API resources
app.add_resource(User, "/")

if __name__ == "__main__":
    # run the server
    app.run(debug=True, port=5000)



import mysql.connector.pooling
import logging
from vvecon.rest_api.utils.Types import NONE, CON


class DBConnection:
    def __init__(self, server_name: str = "", database: str = "", user_name: str = "", password: str = "", attempts: int = 2, delay: int = 100, pool_size: int = 32) -> NONE:
        """
        Setup all the required variables in order to keep the DB connection
        """

        # CONFIGURE DATABASE
        self.server_name = server_name
        self.database = database
        self.user_name = user_name
        self.password = password

        # DEFINE Object Vars
        self.pool_size = pool_size
        self.attempts = attempts
        self.delay = delay
        self.connection_pool = mysql.connector.pooling.MySQLConnectionPool(
            pool_name="pool",
            pool_size=self.pool_size,
            host=self.server_name,
            user=self.user_name,
            password=self.password,
            database=self.database,
        )

    def __enter__(self) -> object:
        """
        Modification for 'with' statement usage
        :return: Return DBConnection object
        """
        return self.connect()

    def __exit__(self, exc_type, exc_val, exc_tb) -> NONE:
        """
        Modification for 'with' statement close eventS
        :param exc_type:
        :param exc_val:
        :param exc_tb:
        :return: Return None, only close the DB & cursor connection
        """
        pass

    def connect(self) -> CON:
        """
        Connect to the Database
        :return: Returns the db cursor if connection established, or else False as it failed
        """
        try:
            return self.connection_pool.get_connection()
        except mysql.connector.Error as e:
            logging.error(e)
            raise e



import requests
from urls import urls
from dotenv import load_dotenv
from os import environ

load_dotenv("../.env")

# API endpoint URL
base = "http://127.0.0.1:5000/"
data = {
    "host": environ.get("USER"),
    "api_key": environ.get("USER_API_KEY")
}

call = "<action:str>"

# Send POST request with data
response = requests.post(base+call, json={**data, **urls[call]})

# Check the status code of the response
if response.status_code == 200:
    # Request successful
    print("Request successful")
elif response.status_code == 201:
    # Resource created
    print("Resource created")
    print("New resource location:", response.headers.get('Location'))
elif response.status_code == 204:
    # No content
    print("Request successful, but no content returned")
elif response.status_code == 400:
    # Bad request
    print("Bad request. Check your syntax or parameters.")
elif response.status_code == 401:
    # Unauthorized
    print("Unauthorized. Authentication required.")
elif response.status_code == 403:
    # Forbidden
    print("Forbidden. You do not have sufficient permissions.")
elif response.status_code == 404:
    # Not found
    print("Resource not found. Verify the URL or parameters.")
elif response.status_code == 500:
    # Internal server error
    print("Internal server error. Please try again later.")
else:
    # Other status codes
    print("Request failed with status code:", response.status_code)
print("Response content:", response.json())



from flask_socketio import emit
from vvecon.rest_api.utils.Types import NONE, BONE
from vvecon.rest_api.utils.Abort import Abortion as VAbortion


class Abortion:
    code: int = 404
    message: str = None
    codes: list = [200, 201, 400, 401, 402, 403, 404, 405, 409, 500, 503]

    def __init__(self, message: str, code: int = 404) -> NONE:
        self.__validate_code__(code)
        self.code = code
        self.message = message

    def __validate_code__(self, code: int) -> BONE:
        if code not in self.__class__.codes:
            raise Exception("VRA001 Error: Invalid return code,", code)
        return True

    def emit(self, sid: str) -> NONE:
        emit('abort', {'code': self.code, 'message': self.message}, room=sid)


def default_emit(result, sid: str) -> NONE:
    if type(result) is Abortion:
        result.emit(sid)
    if type(result) is VAbortion:
        Abortion(result.message, result.code).emit(sid)


def default_empty_emit(result, abortion: Abortion, sid: str) -> NONE:
    if type(result) is bool and result is False:
        abortion.emit(sid)
    if type(result) is Abortion:
        result.emit(sid)


def emit_wrong_host(sid: str) -> NONE:
    Abortion("Invalid host information.", 403).emit(sid)


def emit_wrong_api_key(sid: str) -> NONE:
    Abortion("Incorrect api key.", 401).emit(sid)


def emit_invalid_method(sid: str) -> NONE:
    Abortion("Requested method not found.", 405).emit(sid)


def emit_controller_missing_arg(arg, sid: str) -> NONE:
    Abortion(arg+": Controller missing an argument on parser.", 400).emit(sid)


def emit_controller_arg_type_error(arg, sid: str) -> NONE:
    Abortion(arg+": Controller argument type doesn't match with the parser.", 400).emit(sid)



from flask_mail import Mail as flaskMail, Message


class Mail(flaskMail):
    def __init__(self, *args, **kwargs):
        super(Mail, self).__init__(*args, **kwargs)

    def send(self, recipients: list, subject: str = "", body: str = ""):
        msg = Message(
            subject=subject,
            sender=self.app.config['EMAIL'],
            recipients=recipients
        )
        msg.html = body
        super(Mail, self).send(msg)
        return True



from flask import request
from flask_socketio import Namespace
from vvecon.rest_api.socket.SockAbort import *
from vvecon.rest_api.utils.Types import ANY_, DICT
import inspect


class Socket(Namespace):
    # To be defined
    API_HOSTS: list = None  # all the hosts which has access to the resource of Admin
    API_KEYS: dict = None  # api keys by their host
    ARGS_PARSER = None  # Initializing parser
    CONTROLLER = None  # initialize controller
    namespace_name = None  # initialize Namespace

    def filter_args(self, func: str, args: dict) -> dict:
        """
        This filter the args dictionary according to the need of arguments required by the controller function.
        :param func: Name of controller function
        :param args: arguments given by api
        :return: returns a filtered dictionary fits for the controller function
        """
        data = dict()
        params = inspect.signature(self.__class__.CONTROLLER.__getattribute__(func)).parameters
        for param in params.values():
            if str(param.default) == "<class 'inspect._empty'>" and param.name not in args and param.annotation != str:
                emit_controller_missing_arg(param.name, args['socket_id'])
            if param.name not in args and param.annotation == str:
                data[param.name] = ""
            if param.name in args and type(args[param.name]) != param.annotation:
                emit_controller_arg_type_error(param.name, args['socket_id'])
            if param.name in args:
                data[param.name] = args[param.name]
        return data

    # validate the host
    def validate_host(self, host: str, api_key: str, socket_id: str) -> bool:
        # check if the host is known
        if host not in self.__class__.API_HOSTS:
            emit_wrong_host(socket_id)
            return False
        # check if the aoi key is correct for the current host
        if api_key != self.__class__.API_KEYS[host]:
            emit_wrong_api_key(socket_id)
            return False
        return True

    @staticmethod
    def check(arg):
        return type(arg) == Abortion

    @staticmethod
    def failed(arg):
        default_emit(arg, request.sid)
        return False

    def main(self, action: str, req, proceed=True) -> DICT:
        # bind action with '_'
        func = "_".join(action.split(" "))
        # parsing arguments from header
        if hasattr(self.ARGS_PARSER, func):
            args = self.__class__.ARGS_PARSER.__getattribute__(action).parse_args(req, strict=False)
        # parse with the default parser if no dedicated parser
        else:
            args = self.ARGS_PARSER.default_socket_require_args.parse_args(strict=False)

        if 'socket_id' not in args:
            args['socket_id'] = request.sid

        if self.check(args):
            return self.failed(args)

        # check if the host is authorized
        valid_host = self.validate_host(args['host'], args['api_key'], request.sid)
        if self.check(valid_host):
            return self.failed(valid_host)

        if proceed:
            return self.__default__(func, args)
        return args

    def __default__(self, func: str, args: dict) -> ANY_:
        result = self.__class__.CONTROLLER.__getattribute__(func)(**self.filter_args(func, args))
        if result is not False:
            print("FUN:", func, "\nRES:", result)
            default_emit(result, args['socket_id'])
            if self.check(result):
                return self.failed(result)
            return result
        return False

    @staticmethod
    def load_args(data):
        data['socket_id'] = request.sid
        return data

    def on_connect(self, data):
        # try:
        #     print("GOT HERE", request.sid)
        #     self.emit("connect", room=request.sid, namespace=self.namespace_name)
        # except Exception as e:
        #     print("CON ERR", e)
        pass

    def on_login(self, data):
        args = self.load_args(data)
        print("LOGGED IN")
        if not self.check(args) and self.main("connect", args):
            emit("login", {"message": "You are logged in", "socket_id": args['socket_id']},
                 room=args['socket_id'], namespace=self.namespace_name)

    def on_logout(self, data):
        args = self.load_args(data)
        if not self.check(args):
            self.main("disconnect", args)

    def on_disconnect(self):
        try:
            self.disconnect(request.sid, self.namespace_name)
        except Exception as e:
            print("DIS ERR", e)



from vvecon.rest_api.utils import Abortion
from vvecon.rest_api.utils.Types import ALL_, DICT, BOOL, CURSOR, CON, LIST, ANY
import functools
from urllib.parse import unquote, quote
from mysql.connector.errors import ProgrammingError, InterfaceError, OperationalError, InternalError

__all__ = ("Model", "cover")


def cover(error: str):
    """
    This method is to handle errors & exceptions occurs
    :param error: Preferred error message if the process fails, that can be delivered to API client
    :return: Return either whatever the decorated function returns or the error message
    """

    def func(f):
        """
        This method will wrap the desired function with a wrapping tool
        :param f: Function to be wrapped around
        :return: Returns an object for whatever returning from the decorated function
        """

        @functools.wraps(f)
        def wrapper(*args, **kwargs):
            """
            This wrapper function will cover the function with base flow of the process
            :param args: arguments to be passed into the function
            :param kwargs: keywords to be passed into the function
            :return: Returns whatever returning from the decorated function
            """
            try:
                return f(*args, **kwargs)
            except (ProgrammingError, InterfaceError, InternalError, OperationalError, IndexError) as e:
                print("[ERROR: " + error.upper() + "]", e)
                return Abortion("SYSTEM:Something went wrong, " + error.lower())

        return wrapper

    return func


# -- Model -- #
class Model:

    @staticmethod
    def unquote(argument: ALL_) -> ALL_:
        return unquote(argument) if isinstance(argument, str) else argument

    @staticmethod
    def quote(**kwargs):
        return {key: quote(value) if isinstance(value, str) else value for key, value in kwargs.items()}

    @staticmethod
    def parse(dictionary: dict, key: str):
        return dictionary[key] if key in dictionary.keys() else ""

    def build_condition(self, args: LIST = None, quote_args: LIST = None):
        conditions = []
        if args is not None:
            conditions.append(*[self.validate_input(*arg) for arg in args if type(arg) == list and len(arg) == 3])
        if quote_args is not None:
            conditions.append(
                *[self.validate_input(
                    arg[0], arg[1], quote(arg[2]) if isinstance(arg[2], str) else arg[2])
                    for arg in quote_args if type(arg) == list and len(arg) == 3]
            )
        return "AND".join(self.clens(conditions)) if len(conditions) else ""

    @staticmethod
    def validate_input(alias: str, column_name: str, value: ANY) -> str:
        return f"{alias}.`{column_name}` = '{value}'" if (
                value is not None and value != "" and value is not False and value != 0) else ""

    @staticmethod
    def clens(data: list) -> list:
        while "" in data:
            data.remove("")
        return data

    @staticmethod
    def get(cursor: CURSOR) -> BOOL:
        """
        This method gives True if the sql has returned a positive data or else False
        :param cursor: Database connection cursor
        :return: Returns True if the result contain positive data or else False
        """
        return False if len(cursor.fetchall()) == 0 else True

    def get_one(self, cursor: CURSOR) -> ALL_:
        """
        This method gives reserved value which was taken through sql code or else False if empty
        :param cursor: Database connection cursor
        :return: Returns a single data (One value) or else False
        """
        result = cursor.fetchall()
        return False if len(result) == 0 else self.unquote(result[0][0])

    def get_row_data(self, cursor: CURSOR) -> DICT:
        """
        This method gives reserved data dictionary or else False if empty
        :param cursor: Database connection cursor
        :return: Returns a set of data dictionary (One row) or else False if empty
        """
        result = cursor.fetchall()
        if len(result) == 0:
            return False
        columns = cursor.description
        return {columns[index][0]: self.unquote(value) for index, value in enumerate(result[0])}

    def get_column_data(self, cursor: CURSOR) -> LIST:
        """
        This method gives reserved array of data containing a single column information
        :param cursor: Database connection cursor
        :return: Returns an array of data (One Column) or else False if empty
        """
        result = cursor.fetchall()
        if len(result) == 0:
            return False
        return [self.unquote(value[0]) for value in result]

    def get_dict(self, cursor: CURSOR) -> DICT:
        """
        This method gives a dictionary of data of key and value columns combined or else False if empty
        :param cursor: Database connection cursor
        :return: Returns a dictionary of key, value column data or else False if empty
        """
        result = cursor.fetchall()
        if len(result) == 0:
            return False
        return {key: self.unquote(value) for key, value in result}

    def get_data(self, cursor: CURSOR) -> LIST:
        """
        This method gives reserved array of data dictionaries or else False if empty
        :param cursor: Database connection cursor
        :return: Returns a set of array contains data dictionaries (Many rows) or else False if empty
        """
        result = cursor.fetchall()
        if len(result) == 0:
            return False
        columns = cursor.description
        return [{columns[index][0]: self.unquote(value) for index, value in enumerate(result[row])}
                for row in range(len(result))]

    @staticmethod
    def commit(con: CON) -> BOOL:
        """
        This method commit the data to the MySQL server
        :param con: Database connection
        :return: Returns True if the committing succeed
        """
        con.commit()
        return True

    def execute(self, cursor: CURSOR, query: str, args: DICT = None, quote_args: DICT = None):
        if args is None:
            args = {}
        if quote_args is None:
            quote_args = {}
        cursor.execute(query.format(**args, **self.quote(**quote_args)))



import mysql
import logging
from mysql import connector
from vvecon.rest_api.utils.Types import NONE, CON


class DBConnection:
    def __init__(self, server_name: str = "", database: str = "", user_name: str = "", password: str = "",
                 attempts: int = 2, delay: int = 100, pool_size: int = 32) -> NONE:
        """
        Setup all the required variables in order to keep the DB connection
        """

        # CONFIGURE DATABASE
        self.server_name = server_name
        self.database = database
        self.user_name = user_name
        self.password = password

        # DEFINE Object Vars
        self.attempts = attempts
        self.delay = delay
        self.pool_size = pool_size
        self.connection = None

        self.connect()

    def __enter__(self) -> CON:
        """
        Modification for 'with' statement usage
        :return: Return DBConnection object
        """
        self.ping()
        return self.connection

    def __exit__(self, exc_type, exc_val, exc_tb) -> NONE:
        """
        Modification for 'with' statement close eventS
        :param exc_type:
        :param exc_val:
        :param exc_tb:
        :return: Return None, only close the DB & cursor connection
        """
        pass

    def connect(self) -> NONE:
        """
        Connect to the Database
        :return: Returns the connection cursor if connection established, or else False as it failed
        """
        try:
            self.connection = mysql.connector.connect(
                host=self.server_name,
                user=self.user_name,
                password=self.password,
                database=self.database,
            )
        except mysql.connector.Error as e:
            logging.error(e)
            raise e

    def ping(self) -> NONE:
        """
        This method is to reestablish the DB connection
        :return: None
        """
        if self.connection and self.connection.is_connected():
            self.connection.ping(reconnect=True, attempts=self.attempts, delay=self.delay)
        else:
            self.connect()



from vvecon.rest_api.connections import MySQLDBConnection
from vvecon.rest_api.connections import MySQLDBSingleConnection
from vvecon.rest_api.utils.Types import CON

__all__ = ("MySQLDBConnection", "MySQLDBSingleConnection", "get_connection")


def get_connection(server_name: str = "", database: str = "", user_name: str = "", password: str = "",
                   system: str = "mysql", method: str = "single", attempts: int = 2, delay: int = 100,
                   pool_size: int = 32) -> CON:
    connections = {
        "mysql": {
            "multi": MySQLDBConnection,
            "single": MySQLDBSingleConnection
        }
    }

    if system in connections:
        if method in connections[system]:
            return connections[system][method].DBConnection(server_name, database, user_name, password, attempts, delay,
                                                            pool_size)
        else:
            raise AttributeError("Invalid method")
    else:
        raise AttributeError("Invalid system")



from flask import Flask
from flask_restful import Api
from flask_cors import CORS

from vvecon.rest_api.socket import SocketIO
from vvecon.rest_api.utils import Mail
from vvecon.rest_api.utils.Types import NONE


class App(Flask):
    SOCK = None
    MAIL = None

    def __init__(self, *args, **kwargs) -> NONE:
        # initialize Flask Rest API
        super(App, self).__init__(*args, **kwargs)
        self.CORS = CORS(self)
        self.API = Api(self)

    def add_resource(self, resource, *urls, **kwargs) -> NONE:
        self.API.add_resource(resource, *[url+"<string:action>" for url in urls], **kwargs)

    def mail(self, mail: Mail, email: str, password: str) -> NONE:
        self.__class__.MAIL = mail
        self.__class__.MAIL.init_app(self)
        self.config['MAIL_SERVER'] = 'smtp.gmail.com'
        self.config['MAIL_PORT'] = 465
        self.config['MAIL_USERNAME'] = email
        self.config['MAIL_PASSWORD'] = password
        self.config['MAIL_USE_TLS'] = False
        self.config['MAIL_USE_SSL'] = True

    def socket(self, sock: SocketIO, mode=""):
        """
        :params sock: SocketIO object which defines in Resources
        :params mode: async modes - eventlet
        """
        self.__class__.SOCK = sock
        if mode != "":
            self.__class__.SOCK.init_app(self, async_mode=mode)
        if mode == "":
            self.__class__.SOCK.init_app(self)

    def run(self, debug: bool = False, port: int = 5000, **kwargs):
        if self.__class__.SOCK is not None:
            super(App, self).run(debug=debug, port=port, **kwargs)
            return
        super(App, self).run(debug=debug, port=port)



from vvecon.rest_api.utils import Mail
from vvecon.rest_api.utils.Controller import Controller, cover
from vvecon.rest_api.utils.Types import NONE, OBJ_, ANY, CURSOR, CON
from Model.UserModel import UserModel
import pytz


# -- GLOBAL VARS -- #
TIME_ZONE = pytz.timezone('America/New_York')


# -- User Controller -- #
class UserController(Controller):
    def __init__(self, db: OBJ_, mail: Mail = None) -> NONE:
        # initialize database connections
        self.db = db
        # initialize mail service
        self.__class__.MAIL = mail
        # initialize user model
        self.user_model = UserModel()



from vvecon.rest_api.socket.Socket import Socket
from vvecon.rest_api.socket.SocketIO import SocketIO
from vvecon.rest_api.socket.SockParser import SockParser, SocketParser

__all__ = ("Socket", "SockParser", "SocketParser", "SocketIO")



import os
from ftplib import FTP
from dotenv import load_dotenv
from os import environ

load_dotenv("../.env")

# FTP server credentials and file information
FTP_URL = environ.get("FTP_URL")
FTP_BASE = environ.get("FTP_BASE")
FTP_HOST = environ.get("FTP_HOST")
FTP_USER = environ.get("FTP_USER")
FTP_PASSWORD = environ.get("FTP_PASSWORD")
LOCAL_BASE = environ.get("LOCAL_BASE")
FILES = ['.env', 'main.py', 'Resources.py']
FOLDERS = ['Controller', 'Model', 'Parsers']

if __name__ == "__main__":
    try:

        print("CONNECTING")

        # Create an FTP connection
        ftp = FTP(FTP_HOST)

        # Login to the FTP server
        ftp.login(user=FTP_USER, passwd=FTP_PASSWORD)

        print("CONNECTED TO THE SERVER")

        # Change the working directory (if needed)
        # ftp.cwd("/remote_directory")


        def upload(path):
            # Open the local file in binary mode for uploading
            with open(LOCAL_BASE + path, "rb") as File:
                # Use the STOR command to upload the file
                ftp.storbinary("STOR " + FTP_BASE + path, File)


        print("UPLOADING FILES")
        for FILE in FILES:
            upload(FILE)

        for FOLDER in FOLDERS:
            files = os.listdir(LOCAL_BASE + FOLDER)
            print("UPLOADING FILES ON " + FOLDER)
            for file in files:
                if file.endswith(".py"):
                    upload(FOLDER + "/" + file)

        print("UPLOAD COMPLETED")
        # Close the FTP connection
        ftp.quit()

    except Exception as e:
        print("CONNECTION FAILED\nDUE TO\n", e)



from vvecon.rest_api.utils.Model import Model, cover
from vvecon.rest_api.utils.Types import CURSOR, CON, ALL_
from vvecon.rest_api.utils import Abortion
import pytz


# -- User Model -- #
class UserModel(Model):
    TIME_ZONE = pytz.timezone('America/New_York')



from typing import MutableSequence
from flask import request, current_app
from flask_restful.reqparse import RequestParser, Argument
from vvecon.rest_api.utils.Types import NONE, str__, OBJET, LIST
from vvecon.rest_api.socket.SockAbort import Abortion
import six


_friendly_location = {
    u'json': u'the JSON body',
    u'form': u'the post body',
    u'args': u'the query string',
    u'values': u'the post body or the query string',
    u'headers': u'the HTTP headers',
    u'cookies': u'the request\'s cookies',
    u'files': u'an uploaded file',
}


class SocketArgument(Argument):
    unparsed_arguments = {}

    def source(self, req):
        return req

    def parse(self, req, bundle_errors=False):
        """Parses argument value(s) from the request, converting according to
        the argument's type.

        :param req: The flask request object to parse arguments from
        :param bundle_errors: Do not abort when first error occurs, return a
            dict with the name of the argument and the error message to be
            bundled
        """
        source = self.source(req)
        self.unparsed_arguments = req

        results = []

        # Sentinels
        _not_found = False
        _found = True

        for operator in self.operators:
            name = self.name + operator.replace("=", "", 1)
            if name in source:
                # Account for MultiDict and regular dict
                if hasattr(source, "getlist"):
                    values = source.getlist(name)
                else:
                    values = source.get(name)
                    if not (isinstance(values, MutableSequence) and self.action == 'append'):
                        values = [values]

                for value in values:
                    if hasattr(value, "strip") and self.trim:
                        value = value.strip()
                    if hasattr(value, "lower") and not self.case_sensitive:
                        value = value.lower()

                        if hasattr(self.choices, "__iter__"):
                            self.choices = [choice.lower()
                                            for choice in self.choices]
                    try:
                        value = self.convert(value, operator)
                    except Exception as error:
                        if self.ignore:
                            continue
                        return self.handle_validation_error(error, bundle_errors)

                    if self.choices and value not in self.choices:
                        if current_app.config.get("BUNDLE_ERRORS", False) or bundle_errors:
                            return self.handle_validation_error(
                                ValueError(u"{0} is not a valid choice".format(
                                    value)), bundle_errors)
                        self.handle_validation_error(
                                ValueError(u"{0} is not a valid choice".format(
                                    value)), bundle_errors)

                    if name in self.unparsed_arguments:
                        self.unparsed_arguments.pop(name)
                    results.append(value)

        if not results and self.required:
            if isinstance(self.location, six.string_types):
                error_msg = u"Missing required parameter in {0}".format(_friendly_location.get(self.location,
                                                                                               self.location))
            else:
                friendly_locations = [_friendly_location.get(loc, loc) for loc in self.location]
                error_msg = u"Missing required parameter in {0}".format(' or '.join(friendly_locations))
            if current_app.config.get("BUNDLE_ERRORS", False) or bundle_errors:
                return self.handle_validation_error(ValueError(error_msg), bundle_errors)
            return self.handle_validation_error(ValueError(error_msg), bundle_errors)

        if not results:
            if callable(self.default):
                return self.default(), _not_found
            else:
                return self.default, _not_found

        if self.action == 'append':
            return results, _found

        if self.action == 'store' or len(results) == 1:
            return results[0], _found
        return results, _found

    def handle_validation_error(self, error, bundle_errors):
        """Called when an error is raised while parsing. Aborts the request
        with a 400 status and an error message

        :param error: the error that was raised
        :param bundle_errors: do not abort when first error occurs, return a
            dict with the name of the argument and the error message to be
            bundled
        """
        error_str = six.text_type(error)
        error_msg = self.help.format(error_msg=error_str) if self.help else error_str
        msg = {self.name: error_msg}

        if current_app.config.get("BUNDLE_ERRORS", False) or bundle_errors:
            return error, msg
        return Abortion(msg[0] if type(msg) is list else str(msg), code=400), False


class SockParser(RequestParser):
    unparsed_arguments = {}

    def __init__(self, socket_id: bool = True) -> NONE:
        super(SockParser, self).__init__()
        self.argument_class = SocketArgument
        self.add_arg("host", str__)
        self.add_arg("api_key", str__)
        if socket_id:
            self.add_arg("socket_id", str__)

    def add_argument(self, *args, **kwargs):
        """Adds an argument to be parsed.

        Accepts either a single instance of Argument or arguments to be passed
        into :class:`Argument`'s constructor.

        See :class:`Argument`'s constructor for documentation on the
        available options.
        """

        if len(args) == 1 and isinstance(args[0], self.argument_class):
            self.args.append(args[0])
        else:
            self.args.append(self.argument_class(*args, **kwargs))

        # Do not know what other argument classes are out there
        if self.trim and self.argument_class is SocketArgument:
            # enable trim for appended element
            self.args[-1].trim = kwargs.get('trim', self.trim)

        return self

    # replacement function for add_argument to simply the procedure
    def add_arg(self, name: str, __type: OBJET, required: bool = True, choices: LIST = None) -> NONE:
        if choices is not None:
            self.add_argument(name, type=__type, help=name + " is required", required=required, trim=True,
                              nullable=(not required), choices=choices)
        if choices is None:
            self.add_argument(name, type=__type, help=name + " is required", required=required, trim=True,
                              nullable=(not required))

    def parse_args(self, req=None, strict=False, http_error_code=400):
        if req is None:
            req = request

        namespace = self.namespace_class()
        self.__class__.unparsed_arguments = req

        # A record of arguments not yet parsed; as each is found
        # among self.args, it will be popped out
        errors = {}
        for arg in self.args:
            value, found = arg.parse(req, self.bundle_errors)
            if isinstance(value, ValueError):
                errors.update(found)
                found = None
            if found or arg.store_missing:
                namespace[arg.dest or arg.name] = value
        if errors:
            return Abortion("Error while parsing args", code=400)

        if strict and req:
            return Abortion('Unknown arguments: %s' % ', '.join(self.__class__.unparsed_arguments.keys()), code=400)
        return namespace


# -- Socket Parser -- #

class SocketParser:
    default_socket_require_args = SockParser()



from vvecon.rest_api.utils.ReqParser import ReqParser, Parser
from vvecon.rest_api.utils.Router import Router
from vvecon.rest_api.utils.Abort import abort, Abortion
from vvecon.rest_api.utils.Mail import Mail

__all__ = ("ReqParser", "Router", "abort", "Abortion", "Parser", "Mail")



from flask_restful import abort
from vvecon.rest_api.utils.Types import NONE, BONE


class Abortion:
    code: int = 404
    message: str = None
    codes: list = [200, 201, 400, 401, 402, 403, 404, 405, 409, 500, 503]

    def __init__(self, message: str, code: int = 404) -> NONE:
        self.__validate_code__(code)
        self.code = code
        self.message = message

    def __validate_code__(self, code: int) -> BONE:
        if code not in self.__class__.codes:
            raise Exception("VRAError 001: Invalid return code,", code)
        return True

    def abort(self) -> NONE:
        abort(self.code, message=self.message)


# -- abort functions -- #

# default abortion for 404 error
def default_abort(result) -> NONE:
    if type(result) is Abortion:
        result.abort()


# default abortion for custom errors
def default_empty_abort(result, abortion: Abortion) -> NONE:
    if type(result) is bool and result is False:
        abortion.abort()
    if type(result) is Abortion:
        result.abort()


# abort if host is incorrect
def abort_wrong_host() -> NONE:
    Abortion("VRAError 002: Invalid host information.", 403).abort()


# abort if api key is incorrect
def abort_wrong_api_key() -> NONE:
    Abortion("VRAError 003: Incorrect api key.", 401).abort()


# abort if requested an invalid method
def abort_invalid_method() -> NONE:
    Abortion("VRAError 004: Requested method not found.", 405).abort()


# abort if failed to provide controller required arguments
def abort_controller_missing_arg(arg) -> NONE:
    Abortion("VRAError 005: "+arg+": Controller missing a argument on parser.", 400).abort()


# abort if controller argument required type doesn't match with the parser provided
def abort_controller_arg_type_error(arg) -> NONE:
    Abortion("VRAError 006: "+arg+": Controller argument type doesn't match with the parser.", 400).abort()



from flask_restful.reqparse import RequestParser
from vvecon.rest_api.utils.Types import LIST, str__, NONE, OBJET

__all__ = ("ReqParser", "Parser")


# -- ReqParser -- #
class ReqParser(RequestParser):
    """
    This is to extend the RequestParser and simply the process for the dev
    """

    def __init__(self) -> NONE:
        super(ReqParser, self).__init__()
        self.add_arg("host", str__)
        self.add_arg("api_key", str__)

    # replacement function for add_argument to simply the procedure
    def add_arg(self, name: str, __type: OBJET, required: bool = True, choices: LIST = None) -> NONE:
        if choices is not None:
            self.add_argument(name, type=__type, help=name + " is required", required=required, trim=True,
                              nullable=(not required), choices=choices)
        if choices is None:
            self.add_argument(name, type=__type, help=name + " is required", required=required, trim=True,
                              nullable=(not required))


# -- Parser -- #
class Parser:
    default_require_args = ReqParser()



from vvecon.rest_api.utils import ReqParser, Parser
from vvecon.rest_api.utils.Types import str__, int__


# -- User Parser -- #
class UserParser(Parser):
    """
    This class is to define requirement parses and simply the process
    """



import os.path
from os import environ
from dotenv import load_dotenv
from vvecon.rest_api.connections import get_connection

# load ENV
load_dotenv('../.env')
# initialize database connection
server_name = environ.get("SERVER")
database = environ.get("DATABASE")
user_name = environ.get("USER_NAME")
password = environ.get("PASSWORD")
db = get_connection(server_name, database, user_name, password)

folder = "Backup"
tables = []


def backup(table):
    with db as con:
        cursor = con.cursor()
        query = """INSERT INTO\n`{table_name}`\n""".format(table_name=table)

        cursor.execute(f"SELECT * FROM {table}")
        columns = [column_data[0] for column_data in cursor.description]
        query += """({columns})\nVALUES\n""".format(
            columns=", ".join(["`{column}`".format(column=column) for column in columns])
        )
        result = cursor.fetchall()
        query += ",\n".join(["({record})".format(
            record=", ".join(["'{data}'".format(data=data) for data in record])) for record in result]
        )
        query += ";"
        print(query)
        if len(result) > 0:
            if not os.path.isfile(f"{folder}/{table}.sql"):
                with open(f"{folder}/{table}.sql", 'x', encoding='utf-8') as File:
                    File.write(query)
            else:
                with open(f"{folder}/{table}.sql", 'w', encoding='utf-8') as File:
                    File.write(query)


if __name__ == "__main__":
    with db as con:
        cursor = con.cursor()
        cursor.execute("""
        SHOW TABLES;
        """)
        result = cursor.fetchall()
        tables = [table_data[0] for table_data in result]

    for table in tables:
        backup(table)



from vvecon.rest_api.connections import get_connection
import re
from dotenv import load_dotenv
from os import environ, listdir

global columns

# load ENV
load_dotenv('../.env')
# initialize database connection
server_name = environ.get("SERVER")
database = environ.get("DATABASE")
user_name = environ.get("USER_NAME")
password = environ.get("PASSWORD")
db = get_connection(server_name, database, user_name, password)

folder = "Tables"
local = "Local"

files = listdir(folder)
local_files = listdir(local)


def table_exists(cursor, table):
    cursor.execute("""
    SHOW TABLES
    LIKE '{table}';
    """.format(table=table))
    return False if len(cursor.fetchall()) == 0 else True


def column_exists(cursor, table, column):
    cursor.execute("""
    SHOW COLUMNS
    FROM `{table}`
    LIKE '{column}';
    """.format(table=table, column=column))
    return False if len(cursor.fetchall()) == 0 else True


def create(query, table):
    try:
        with db as con:
            cursor = con.cursor()
            if not table_exists(cursor, table):
                print("'"+table+"' TABLE DOES NOT EXISTS.")
                cursor.execute(query)
                con.commit()
                print("'"+table+"' TABLE CREATED.")
    except Exception as e:
        print(e)


def alter(line, table):
    global columns
    try:
        with db as con:
            cursor = con.cursor()
            column = re.search(r"`(.*?)`", line).group(1)
            columns.append(column)
            if not column_exists(cursor, table, column):
                print("'"+column+"' COLUMN DOES NOT EXISTS ON TABLE '"+table+"'.")
                cursor.execute("""
                ALTER TABLE `{table}`
                ADD COLUMN {line};
                """.format(table=table, line=line[:-1]))
                print("'"+column+"' COLUMN CREATED")
            else:
                cursor.execute("""
                ALTER TABLE `{table}`
                MODIFY COLUMN {line};
                """.format(table=table, column=column, line=line[:-1]))
            con.commit()
    except Exception as e:
        print(e)


def primary(line, table):
    try:
        with db as con:
            cursor = con.cursor()
            column = re.search(r"`([^`]+)`", line).group(1)
            if column_exists(cursor, table, column):
                cursor.execute("SET FOREIGN_KEY_CHECKS=0;")
                cursor.execute("""
                ALTER TABLE `{table}`
                DROP PRIMARY KEY,
                ADD PRIMARY KEY (`{column}`);
                """.format(table=table, column=column))
                cursor.execute("SET FOREIGN_KEY_CHECKS=1;")
    except Exception as e:
        print(e)


def route(line, query, table):
    line = line.strip()
    if line.startswith('CREATE'):
        create(query, table)
    if line.startswith("`"):
        alter(line, table)
    if line.startswith('PRIMARY'):
        primary(line, table)


def remove(cursor, con, table, column):
    global columns
    print("INVALID '"+column+"' COLUMN ON TABLE '"+table+"'.")
    cursor.execute("""
    ALTER TABLE `{table_name}`
    DROP COLUMN `{column_name}`;
    """.format(table_name=table, column_name=column))
    con.commit()
    print("'" + column + "' COLUMN REMOVED")


def remove_false_columns(table):
    global columns
    with db as con:
        cursor = con.cursor()
        cursor.execute("""
        SHOW COLUMNS
        FROM `{table_name}`;
        """.format(table_name=table))
        result = cursor.fetchall()
        table_columns = [column_data[0] for column_data in result]
        for column in table_columns:
            if column not in columns:
                remove(cursor, con, table, column)


def truncate(table):
    with db as con:
        cursor = con.cursor()
        cursor.execute("TRUNCATE `{table_name}`".format(table_name=table))
        con.commit()


def merge(query):
    with db as con:
        cursor = con.cursor()
        cursor.execute(query)
        con.commit()


if __name__ == "__main__":
    print("CHECKING TABLES")
    for file in files:
        if file.endswith('.sql'):
            with open(f"{folder}/{file}", 'r') as File:
                query = File.read().strip()
                lines = query.split('\n')
                table_match = re.search(r"CREATE TABLE `([^`]+)`", lines[0])
                if table_match:
                    table = table_match.group(1)
                    columns = []
                    for line in lines:
                        route(line, query, table)
                    remove_false_columns(table)

    print("UPDATING DATA ON LOCAL BACKUP")
    for file in local_files:
        with open(f"{local}/{file}", 'r') as File:
            query = File.read().strip()
            lines = query.split('\n')
            table_match = re.search(r"`([^`]+)`", lines[1])
            if table_match:
                table = table_match.group(1)
                truncate(table)
                merge(query)



from vvecon.rest_api.utils import Router, Mail
from vvecon.rest_api.socket import SocketIO
from vvecon.rest_api.connections import get_connection
from Parsers.UserParser import UserParser
from Controller.UserController import UserController
from dotenv import load_dotenv
from os import environ

sock = SocketIO()
mail = Mail()

__all__ = ['User', 'sock', 'mail']

# load ENV
load_dotenv()

# initialize database connection
server_name = environ.get("SERVER")
database = environ.get("DATABASE")
user_name = environ.get("USER_NAME")
password = environ.get("PASSWORD")
db = get_connection(server_name, database, user_name, password)


# -- HOSTS -- #
USER = environ.get("USER")


# -- User Router -- #
class User(Router):
    # api keys by their host
    API_KEYS = {
        USER: environ.get("USER_API_KEY")
    }

    # pre created privileges attribute
    PRIVILEGES = {
        USER: {
            "example"
        }
    }

    # Initializing parser
    ARGS_PARSER = UserParser()

    # initialize controller
    CONTROLLER = UserController(db, mail)



import os
from ftplib import FTP
from dotenv import load_dotenv
from os import environ

load_dotenv("../.env")

# FTP server credentials and file information
FTP_URL = environ.get("WEB_FTP_URL")
FTP_BASE = environ.get("WEB_FTP_BASE")
FTP_HOST = environ.get("WEB_FTP_HOST")
FTP_USER = environ.get("WEB_FTP_USER")
FTP_PASSWORD = environ.get("WEB_FTP_PASSWORD")
LOCAL_BASE = environ.get("WEB_LOCAL_BASE")
FILES = ['.env', 'index.php']
FOLDERS = ['config', 'controller', 'core', 'model', 'paths', 'view', 'statics']

if __name__ == "__main__":
    try:

        print("CONNECTING")

        # Create an FTP connection
        ftp = FTP(FTP_HOST)

        # Login to the FTP server
        ftp.login(user=FTP_USER, passwd=FTP_PASSWORD)

        print("CONNECTED TO THE SERVER")

        # Change the working directory (if needed)
        # ftp.cwd("/remote_directory")

        def create_remote_directory(remote_dir):
            try:
                ftp.cwd(remote_dir)
            except Exception:
                try:
                    print("MAKE NEW DIR", remote_dir)
                    ftp.mkd(remote_dir)
                except Exception:
                    print("FOLDER ALREADY EXISTS")


        def upload(path):
            # Open the local file in binary mode for uploading
            with open(LOCAL_BASE + path, "rb") as File:
                # Use the STOR command to upload the file
                ftp.storbinary("STOR " + FTP_BASE + path, File)


        print("UPLOADING FILES")
        for FILE in FILES:
            upload(FILE)


        def upload_folders(FOLDER):
            files = os.listdir(LOCAL_BASE + FOLDER)
            print("UPLOADING FILES ON " + FOLDER)
            for file in files:
                if "." in file:
                    print("UPLOADING", FOLDER + "/" + file)
                    upload(FOLDER + "/" + file)
                else:
                    create_remote_directory(FOLDER + "/" + file)
                    upload_folders(FOLDER + "/" + file)
            return None


        for FOLDER in FOLDERS:
            upload_folders(FOLDER)

        print("UPLOAD COMPLETED")
        # Close the FTP connection
        ftp.quit()

    except Exception as e:
        print("CONNECTION FAILED\nDUE TO\n", e)



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

long_description = "A RESTFul API Framework for fast, easy and secure development By VVECON Developers"

VERSION = '1.0.4'
DESCRIPTION = 'RESTFul API Framework'

# Setting up
setup(
    name="vvecon",
    version=VERSION,
    author="VVECON Developers(Sasindu Sulochana)",
    author_email="<vveconllc@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/plain",
    long_description=long_description,
    packages=find_packages(),
    install_requires=['Flask==2.3.2', 'Flask-RESTful==0.3.9', "Flask-Cors==3.0.10", "Flask-SocketIO==5.3.5", "Flask-Mail==0.9.1", "python-socketio==5.8.0", "mysql-connector-python==8.0.28", "six==1.16.0", "urllib3==1.26.8", "python-dotenv==1.0.0", "pytz==2021.3", "MarkupSafe~=2.1.1", "Werkzeug==2.3.6", "httpagentparser~=1.9.3", "greenlet==3.0.2", "eventlet==0.33.3"],
    keywords=['vvecon', 'python', 'rest', 'api', 'restful', 'sasindu', 'sulochana', 'socket'],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


from flask_restful import Resource
from vvecon.rest_api.utils import Parser, Controller
from vvecon.rest_api.utils.Types import ANY_
from vvecon.rest_api.utils.Abort import *
from urllib.parse import unquote
from flask import jsonify
from flask_cors import cross_origin
import inspect


class Router(Resource):
    # To be defined
    API_KEYS: dict = None  # api keys by their host
    PRIVILEGES: dict = None  # pre created privileges attribute
    ARGS_PARSER: Parser = None  # Initializing parser
    CONTROLLER: Controller = None  # initialize controller

    ALL_PRIVILEGES = None  # will be created automatically when called

    def __init__(self) -> NONE:
        super(Router, self).__init__()
        self.__class__.ALL_PRIVILEGES = self.get_all_privileges()

    def get_all_privileges(self) -> list:
        # return list(set(key for inner_dict in self.__class__.PRIVILEGES.values() for key in inner_dict.keys() if key))
        return list(set(privilege for host in self.__class__.PRIVILEGES for privilege in self.PRIVILEGES[host]))

    def filter_args(self, func: str, args: dict) -> dict:
        """
        This filter the args dictionary according to the need of arguments required by the controller function.
        :param func: Name of controller function
        :param args: arguments given by api
        :return: returns a filtered dictionary fits for the controller function
        """
        data = dict()
        params = inspect.signature(self.__class__.CONTROLLER.__getattribute__(func)).parameters
        exceptions = ("cursor", "con")
        for param in params.values():
            if param.name not in exceptions:
                if (str(param.default) == "<class 'inspect._empty'>" and param.name not in args
                        and param.annotation != str):
                    abort_controller_missing_arg(param.name)
                if param.name not in args and param.annotation == str:
                    data[param.name] = ""
                if param.name in args and type(args[param.name]) != param.annotation:
                    abort_controller_arg_type_error(param.name)
                if param.name in args:
                    data[param.name] = args[param.name]
        return data

    # validate the host
    def validate_host(self, host: str, api_key: str) -> None:
        # check if the host is known
        if host not in self.__class__.API_KEYS.keys():
            abort_wrong_host()
        # check if the aoi key is correct for the current host
        if api_key != self.__class__.API_KEYS[host]:
            abort_wrong_api_key()

    # act on post method
    @cross_origin(origins="*")
    def post(self, action: str) -> object:
        # getting string normal form of itself
        action = unquote(action)
        # check if the action is valid
        if action not in self.__class__.ALL_PRIVILEGES:
            abort_invalid_method()
        # bind action with '_'
        func = "_".join(action.split(" "))
        # parsing arguments from header
        if hasattr(self.ARGS_PARSER, func):
            args = self.ARGS_PARSER.__getattribute__(func).parse_args(strict=True)
        # parse with the default parser if no dedicated parser
        else:
            args = self.ARGS_PARSER.default_require_args.parse_args(strict=True)
        # check if the host is authorized
        self.validate_host(args['host'], args['api_key'])
        # check if the action is a privilege of the current host
        if action not in self.__class__.PRIVILEGES[args['host']]:
            abort_invalid_method()
        # take the action and return the results
        if hasattr(self, func):
            if 'args' in inspect.signature(self.__getattribute__(func)).parameters:
                return jsonify(result=self.__getattribute__(func)(args)), 201
            if 'args' not in inspect.signature(self.__getattribute__(func)).parameters:
                return jsonify(result=self.__getattribute__(func)()), 201
        # follow default procedure if not dedicated resource function
        if not hasattr(self, func):
            return jsonify(result=self.__default__(func, args)), 201

    @staticmethod
    def abort(result) -> NONE:
        # TOO OLD getattr(globals(), self.__class__.PRIVILEGES[args['host']][action])(result)
        # OLD self.__class__.PRIVILEGES[args['host']][action](result)
        default_abort(result)

    def __default__(self, func: str, args: dict) -> ANY_:
        result = self.__class__.CONTROLLER.__getattribute__(func)(**self.filter_args(func, args))
        self.abort(result)
        return result



from typing import Union, Any
from mysql.connector.pooling import MySQLConnection
from mysql.connector.cursor import MySQLCursor


# unions & data types
BOOL = bool
NONE: None = None
BONE = Union[bool, None]
ALL_ = Union[bool, int, str, float, None]
ANY_ = Union[bool, int, str, float, None, list, dict]
ANY = Any
LIST = Union[bool, list]
DICT = Union[bool, dict]
OBJ_ = object
OBJET = Union[object, type]
CURSOR = MySQLCursor
CON = MySQLConnection


# This function will validate weather the argument is None, Empty, Or Str
def str__(value: str) -> str:
    if value is None:
        raise ValueError("Value cannot be None")
    if not isinstance(value, str):
        raise ValueError("Invalid argument")
    if not value.strip():
        raise ValueError("Value cannot be empty or whitespace")
    return str(value)


# This function will validate weather the argument is None or Int
def int__(value: int) -> int:
    if value is None:
        raise ValueError("Value cannot be None")
    if not isinstance(value, int):
        raise ValueError("Invalid argument")
    return int(value)


# This function will validate weather the argument is None or Float
def float__(value: float) -> float:
    if value is None:
        raise ValueError("Value cannot be None")
    if not isinstance(value, float):
        raise ValueError("Invalid argument")
    return float(value)



import os
import sys


def create_rest_api(name: str = False):
    try:
        current_dir = os.path.dirname(os.path.abspath(__file__))
        destination_dir = os.getcwd()
        if name:
            destination_dir += "/" + name
            if not os.path.exists(destination_dir):
                try:
                    os.mkdir(destination_dir)
                except FileExistsError:
                    print("FOLDER ALREADY EXISTS")

        def copy_file(file):
            print("CREATING", destination_dir + "/" + file[len(current_dir + "/rest_api/test/sample/"):])
            with open(file, 'rb') as File:
                with open(destination_dir + "/" + file[len(current_dir + "/rest_api/test/sample/"):], 'x'):
                    pass
                with open(destination_dir + "/" + file[len(current_dir + "/rest_api/test/sample/"):], 'wb') as newFile:
                    newFile.write(File.read())

        def copy_folders(folder):
            files = os.listdir(folder)
            for file in files:
                if "." in file:
                    copy_file(folder + "/" + file)
                else:
                    try:
                        print("CREATING FOLDER", destination_dir + "/" + (
                            folder + "/" + file)[len(current_dir + "/rest_api/test/sample/"):]
                        )
                        os.mkdir(destination_dir + "/" + (folder + "/" + file)[
                                                         len(current_dir + "/rest_api/test/sample/"):
                        ])
                    except FileExistsError:
                        pass
                    copy_folders(folder + "/" + file)
            return None

        cdir = current_dir + "/rest_api/test/sample"
        print("WORKING DIRECTORY", cdir)
        copy_folders(cdir)

        with open(destination_dir+"/.env", "x") as File:
            File.write("""# -- Database Environment -- #
SERVER=localhost
DATABASE=example
USER_NAME=root
PASSWORD=


# -- Mail -- #
EMAIL=example@gmail.com
APP_PASSWORD=exampleToken


# -- Hosts & API Keys -- #
DOMAIN=api.example.com

# USER #
USER=user@${DOMAIN}
USER_API_KEY=exampleToken


# -- FTP API -- #
FTP_URL=https://api.example.com
FTP_BASE=/
FTP_HOST=1.1.1.1
FTP_USER=user@api.example.com
FTP_PASSWORD=password
LOCAL_BASE=../


# -- FTP WEB -- #
WEB_FTP_URL=https://example.com
WEB_FTP_BASE=/
WEB_FTP_HOST=1.1.1.1
WEB_FTP_USER=user@example.com
WEB_FTP_PASSWORD=password
WEB_LOCAL_BASE=C://path/to/local/web/""")
        print("SUCCESSFULLY CREATED API!")
    except Exception as e:
        raise Exception("CREATE FAILED:\n", e)


if __name__ == "__main__":
    args = sys.argv
    if len(args) == 3 and args[1] == "create" and args[2] == "rest-api":
        create_rest_api()
    if len(args) == 4 and args[1] == "create" and args[2] == "rest-api":
        create_rest_api(args[3])



import socketio
from dotenv import load_dotenv
from os import environ

load_dotenv("../.env")

# Initialize a Socket.IO client instance
sio = socketio.Client()


# Define a function to be executed when the 'connect' event is received
@sio.on('connect', namespace="/user")
def on_connect():
    user_data = {
        'host': environ.get("USER"),
        'api_key': environ.get("USER_API_KEY"),
    }
    sio.emit('login', user_data, namespace="/user")
    print('Connected to server')


@sio.on('message', namespace='/user')
def on_message(data):
    print("message", data)


@sio.on('abort', namespace='/user')
def on_abort(data):
    print("abort", data)


# Connect to the Socket.IO server
sio.connect('http://ip_addr/user')

# Keep the script running to maintain the connection
sio.wait()
